"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var types_1 = require("@babel/types");
;
var defaultOptions = {
    flag: '&'
};
function pluginExsist(plugins, name) {
    return plugins.some(function (p) { return (Array.isArray(p) ? p[0] : p) === name; });
}
function isClassNameAttr(attr) {
    return !types_1.isJSXSpreadAttribute(attr) && types_1.isJSXIdentifier(attr.name) && attr.name.name === 'className';
}
function getExpectClass(node) {
    var className = node.attributes.find(function (attr) { return isClassNameAttr(attr); });
    if (className) {
        var value = className.value || {};
        if (types_1.isJSXExpressionContainer(value) || types_1.isStringLiteral(value)) {
            return value;
        }
    }
    return null;
}
function isReplacable(node, flagRegexp) {
    if (types_1.isStringLiteral(node) && flagRegexp.test(node.value)) {
        return true;
    }
    return false;
}
function createNewClassValue(parent, current, flagRegexp) {
    if (types_1.isStringLiteral(parent)) {
        var newValueText = current.replace(flagRegexp, parent.value.split(' ')[0]);
        return types_1.stringLiteral(newValueText);
    }
    var expression = parent.expression;
    if (types_1.isTemplateLiteral(expression)) {
        // 没有表达式的末班字符串当做普通字符串处理
        if (expression.expressions.length === 0) {
            var text = expression.quasis[0].value.cooked;
            var newValueText = current.replace(flagRegexp, text.split(' ')[0]);
            return types_1.stringLiteral(newValueText);
        }
    }
    else {
        expression = types_1.templateLiteral([types_1.templateElement({ cooked: '', raw: '' }, false), types_1.templateElement({ cooked: '', raw: '' }, true)], [expression]);
    }
    var newTemplateLiteral = rebuildTemplate(expression, current, flagRegexp);
    return types_1.jsxExpressionContainer(newTemplateLiteral);
}
function rebuildTemplate(template, current, flagRegexp) {
    var matches = matchAll(flagRegexp, current);
    var firstQuaText = template.quasis[0].value.cooked;
    var flagLength = flagRegexp.source.length;
    var expressions = [];
    var quasis = [];
    var index = 0;
    matches.forEach(function (m, i) {
        var expressionsCopy = template.expressions.map(function (e) { return types_1.cloneNode(e); });
        var quasisCopy = template.quasis.map(function (q) { return types_1.cloneNode(q); });
        var left = current.slice(index, m.index);
        index = m.index + flagLength;
        var text = left + firstQuaText;
        if (quasis.length > 0) {
            var tailOfPrev = quasis.pop();
            text = tailOfPrev.value.cooked + text;
        }
        quasisCopy[0] = types_1.templateElement({ cooked: text, raw: text }, false);
        quasis = quasis.concat(quasisCopy);
        expressions = expressions.concat(expressionsCopy);
        if (i === matches.length - 1) {
            // 最后一个
            var tail = quasis.pop();
            var tailText = tail.value.cooked + current.slice(index);
            quasis.push(types_1.templateElement({ cooked: tailText, raw: tailText }, true));
        }
    });
    return types_1.templateLiteral(quasis, expressions);
}
function matchAll(regexp, text) {
    // 手工置零， 防止其他地方使用过
    regexp.lastIndex = 0;
    var matches = [];
    var match;
    while (match = regexp.exec(text)) {
        matches.push(match);
    }
    return matches;
}
function plugin(api) {
    var replaceClassVisitor = {
        JSXAttribute: function (path) {
            if (!isClassNameAttr(path.node)) {
                return;
            }
            var valuePath = path.get('value');
            valuePath.replaceWith(this.newClassValue);
        }
    };
    var classStack = [];
    var index = -1;
    return {
        manipulateOptions: function (opts, parserOpts) {
            if (pluginExsist(parserOpts.plugins, 'typescript') || pluginExsist(parserOpts.plugins, 'jsx')) {
                return;
            }
            parserOpts.plugins.push("jsx");
        },
        visitor: {
            JSXOpeningElement: function (path, state) {
                var node = path.node;
                var classValue = classStack[index];
                var expectValue = getExpectClass(node);
                // 可以被替换 
                if (expectValue) {
                    var flag = Object.assign({}, defaultOptions, state.opts).flag;
                    var flagRegexp = new RegExp(flag, 'g');
                    if (isReplacable(expectValue, flagRegexp) && index > -1) {
                        // 执行替换
                        classValue = createNewClassValue(classValue, expectValue.value, flagRegexp);
                        path.traverse(replaceClassVisitor, { newClassValue: classValue });
                    }
                    else {
                        classValue = expectValue;
                    }
                }
                // non-selfclosing element
                if (!node.selfClosing) {
                    classStack.push(classValue);
                    index++;
                }
            },
            JSXClosingElement: function () {
                classStack.pop();
                index--;
            },
        }
    };
}
exports.default = plugin;
//# sourceMappingURL=transformer.js.map